@@ -3,6 +3,7 @@ package Image::Pngslimmer;
 use 5.008004;
 use strict;
 use warnings;
+use String::CRC32;
 
 require Exporter;
 
@@ -25,9 +26,21 @@ our @EXPORT = qw(
 	
 );
 
-our $VERSION = '0.02';
-
+our $VERSION = '0.03';
 
+sub checkcrc {
+	my $chunk = shift;
+	my ($chunklength, $subtocheck, $generatedcrc, $readcrc);
+	#get length of data
+	$chunklength = unpack("N", substr($chunk, 0, 4));
+	$subtocheck = substr($chunk, 4, $chunklength + 4);
+	$generatedcrc = crc32($subtocheck);
+	$readcrc = unpack("N", substr($chunk, $chunklength + 8, 4));
+	if ($generatedcrc eq $readcrc) {return 1;}
+	#don't match
+	return 0;
+}
+	
 
 sub ispng {
 	my $blob = shift;
@@ -42,14 +55,16 @@ sub ispng {
 	if (substr($blob, 12, 4) ne "IHDR") {
 		return 0;
 	}
+	if (checkcrc(substr($blob, 8)) < 1) {return 0;}
 	my $ihdr_len = unpack("N", substr($blob, 8, 4));	
-	#check for IDAT
+	#check for IDAT - scanning CRCs as we go
 	#scan through all the chunks looking for an IDAT header
 	my $pnglength = length($blob);
 	#start searching from end of IHDR chunk
 	my $searchindex = 16 + $ihdr_len + 4 + 4;
 	my $idatfound  = 0;
 	while ($searchindex < ($pnglength - 4)) {
+		if (checkcrc(substr($blob, $searchindex - 4)) < 1) {return 0;}
 		if (substr($blob, $searchindex, 4) eq "IDAT") {
 			$idatfound = 1;
 			last;
@@ -67,6 +82,8 @@ sub ispng {
 		return 0;
 	}
 	#check for IEND chunk
+	#check CRC first
+	if (checkcrc(substr($blob, $pnglength - 12)) < 1) {return 0;}
 	if (substr($blob, $pnglength - 8, 4) ne "IEND") {
 		return 0;
 	}
@@ -106,6 +123,7 @@ sub discard_noncritical {
 
 sub analyze {
 	my ($blob, $chunk_desc, $chunk_text, $chunk_length, $chunk_CRC, $crit_status, $pub_status, @chunk_array, $searchindex, $pnglength, $nextindex);
+	my ($chunk_CRC_checked);
 	$blob = shift;
 	#is it a PNG?
 	if (Image::Pngslimmer::ispng($blob) < 1){
@@ -124,6 +142,8 @@ sub analyze {
 		$chunk_text = substr($blob, $searchindex, 4);
 		#chunk CRC
 		$chunk_CRC = unpack("N", substr($blob, $searchindex + $chunk_length, 4));
+		#is CRC correct?
+		$chunk_CRC_checked = checkcrc(substr($blob, $searchindex - 4));
 		#critcal chunk?
 		$crit_status = 0;
 		if ((ord($chunk_text) & 0x20) == 0) {$crit_status = 1;}
@@ -132,6 +152,10 @@ sub analyze {
 		if ((ord(substr($blob, $searchindex + 1, 1)) & 0x20) == 0) {$pub_status = 1;}
 		$nextindex = $searchindex - 4;
 		$chunk_desc = $chunk_text." begins at offset $nextindex has data length $chunk_length with CRC $chunk_CRC";
+		if ($chunk_CRC_checked == 1) {
+			$chunk_desc = $chunk_desc." and the CRC is good -";
+		}
+		else {$chunk_desc = $chunk_desc." and there is an ERROR in the CRC -";}
 		if ($crit_status > 0) { $chunk_desc = $chunk_desc." the chunk is critical to the display of the PNG"; }
 		else { $chunk_desc = $chunk_desc." the chunk is not critical to the display of the PNG"; }
 		if ($pub_status > 0) { $chunk_desc = $chunk_desc." and is public\n"; }
@@ -142,6 +166,7 @@ sub analyze {
 	return @chunk_array;
 }
 
+
 1;
 __END__
 
@@ -157,6 +182,7 @@ Image::Pngslimmer - slims (dynamically created) PNGs
 	$ping = ispng($blob)			#is this a PNG? $ping == 1 if it is
 	$newblob = discard_noncritical($blob)  	#discard non critcal chunks and return a new PNG
 	my @chunklist = analyze($blob) 		#get the chunklist as an array
+	
 
 =head1 DESCRIPTION
 
@@ -175,10 +201,11 @@ worthwhile reduction..
 
 discard_noncritical($blob) will call ispng($blob) before attempting to manipulate the
 supplied stream of bytes - hopefully, therefore, avoiding the accidental mangling of 
-JPEGs or other files. ispng is only a very basic check for PNG definition conformity -
+JPEGs or other files. ispng checks for PNG definition conformity -
 it looks for a correct signature, an image header (IHDR) chunk in the right place, looks
 for (but does not check in any way) an image data (IDAT) chunk and checks there is an
-end (IEND) chunk in the right place.
+end (IEND) chunk in the right place. From version 0.03 onwards ispng also checks CRC
+values.
 
 analyze($blob) is supplied for completeness and to aid debugging. It is not called by 
 discard_noncritical but may be used to show 'before-and-after' to demonstrate the savings
